#!/bin/csh -f

## Set Debug Parameter
# DEBUG=1; erase all temporary files
#       0; leave all temporary files
set DEBUG=1

## Set Pixel Scale
set PIXSCALE=0.105 

if ( $#argv != 3 ) then
    echo "Usage : "$0" (FITS data table) (# of dithering point) (Header Name)"
    exit 1
endif

set FTABLE=$1
set NPOINT=$2
set HEADER=`gawk -v hd=$3 'BEGIN{gsub(".fits","",hd);gsub(".imh","",hd);print(hd)}'`
set OUTIM=ql.$HEADER".fits"


set CLLOG=log/cl`date +%y%m%d%H%M%S`.log 
if( !(-e log) ) then 
  mkdir log 
endif 
if( -f $CLLOG ) then 
  rm $CLLOG 
endif 

echo "IRAF Log of $0 $1 $2 $3" > $CLLOG

echo "IRAF log will be written in $CLLOG" 
echo "### Preparation ###"
echo "### Preparation ###" >> $CLLOG
set DITH=20
@ DITH2 = $DITH * 2
set NFRAME=`gawk -v np=$NPOINT 'END{print(NR/np)}' $FTABLE`
echo NFRAME=$NFRAME
gawk '{n=split($1,a,"/");print("ldata/"a[n])}' $FTABLE | sed 's/.fits/.imh/g' > ldata.tbl
sed 's/ldata/mldata/g' ldata.tbl > mldata.tbl
sed 's/.imh/.imh[257:768,257:768]/g' ldata.tbl > pldata.tbl
gawk -v nf=$NFRAME '{if(i<1) print($0); i++; if(i==nf) i=0;}' ldata.tbl > ldata.1.tbl
if ( $NFRAME > 1 ) then
  gawk -v nf=$NFRAME '{if(i>=1) print($0); i++; if(i==nf) i=0;}' ldata.tbl > ldata.2.tbl
endif
sed 's/.imh/.imh[257:768,257:768]/g' ldata.1.tbl > pldata.1.tbl

foreach filename (`cat $FTABLE`) 
  if ( !(-f $filename) ) then
    echo Error: $filename does not exist!
    echo Exiting....
    exit 1
  endif
end

echo "### FITS Header Analysis ###"
echo "### FITS Header Analysis ###" >> $CLLOG 
if ( -f tmphead.log ) then
  \rm tmphead.log 
endif
echo "imhead @"$FTABLE" longheader=yes > tmphead.log" > cl.com
echo "logout" >> cl.com   
cat cl.com | cl >>& $CLLOG  
grep "^SLT-PA" tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' > tmppa.log 
grep "^RA   " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{print($1*3600+$2*60+$3)}' > tmpra.log 
grep "^M2-TYPE" tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' > tmpm2.log 
#grep "^DEC  " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{print(-1*($1*3600+$2*60+$3))}' > tmpdec.log 
grep "^DEC  " tmphead.log | gawk '{print($3)}' | sed "s/'//g" | sed 's/:/ /g' | gawk '{if(gsub("-","-",$1)>0)s=-1;else s=1;print(-1*s*(s*$1*3600+$2*60+$3))}' > tmpdec.log 
set RA=`gawk '{s+=$1}END{print(s/NR)}' tmpra.log` 
set DEC=`gawk '{s+=$1}END{print(s/NR)}' tmpdec.log` 
set PA=`gawk '{s+=$1}END{print(s/NR)}' tmppa.log` 

set SMTYPE=`sed -n 1p tmpm2.log`
if ( $SMTYPE == "CS_IR" ) then
  set PIXSCALE=0.105
else if ( $SMTYPE == "NS_OPT" ) then
  set PIXSCALE=0.111
endif
echo "Secondary Mirror Type is $SMTYPE"
echo "PIXSCALE set to $PIXSCALE (arcsec/pix)"

gawk -v ps=$PIXSCALE -v ra=$RA -v dec=$DEC '{print(($1-ra)*15*cos(dec/3600/180*3.14159)/ps)}' tmpra.log > tmpra2.log 
gawk -v ps=$PIXSCALE -v ra=$RA -v dec=$DEC '{print(($1-dec)/ps)}' tmpdec.log > tmpdec2.log 
paste tmpra2.log tmpdec2.log | gawk -v pa=$PA '{print(-1*(cos(pa)*$1+sin(pa)*$2),-1*(-sin(pa)*$1+cos(pa)*$2))}' > tmpoffset.log 
gawk -v nf=$NFRAME '{s1+=$1;s2+=$2;if(NR%nf==0){printf("%d %d\n",s1/nf,s2/nf);s1=0;s2=0}}' tmpoffset.log | gawk '{if(NR==1){dx=$1;dy=$2}print($1-dx,$2-dy)}'> tmp.offset1
#if ( $DEBUG == 1 ) then 
  \rm tmppa.log tmpra.log tmpdec.log  tmpra2.log tmpdec2.log tmpoffset.log
#endif

echo "### Flat-fielding ###"
echo "### Flat-fielding ###" >> $CLLOG 
echo "imdel @ldata.tbl" > cl.com
echo "imarith @"$FTABLE" / ./flat.fits @ldata.tbl" >> cl.com
echo "logout" >> cl.com 
cat cl.com | cl >>& $CLLOG

echo "### Offset rejection in Quad 2 ###"
echo "### Offset rejection in Quad 2 ###" >> $CLLOG 
echo "offsetrej @ldata.tbl" > cl.com
echo "logout" >> cl.com
cat cl.com | cl >>& $CLLOG

echo "### Preparation for MKSUBGR command ###"
echo "### Preparation for MKSUBGR command ###" >> $CLLOG 
echo "del level.tbl" > cl.com
echo "imstatistics @pldata.1.tbl field=mean fo- > level.tbl" >> cl.com
echo "logout" >> cl.com  
cat cl.com | cl >>& $CLLOG
awk '{s+=$1;i++;}END{print(s/i)}' level.tbl > tmp.txt
echo "imdel sky1,sky2" > cl.com
echo "imdel sky*.imh" >> cl.com
echo "del bkg1.tbl,bkg2.tbl" >> cl.com
echo "logout" >> cl.com  
cat cl.com | cl >>& $CLLOG

echo "### mksubgr ###"
echo "### mksubgr ###" >> $CLLOG 
gawk -v nf=1 '{print("mksubgr @ldata.1.tbl sky1 1 "nf" "$1" 257 768 257 768 bkg1.tbl")}' tmp.txt > cl.com
if( $NFRAME > 1 ) then
  gawk -v nf=$NFRAME '{print("mksubgr @ldata.2.tbl sky2 1 "nf-1" "$1" 257 768 257 768 bkg2.tbl")}' tmp.txt >> cl.com
endif
echo "logout" >> cl.com  
cat cl.com | cl >>& $CLLOG 

echo "### Sky-subtraction ###"
echo "### Sky-subtraction ###" >> $CLLOG 
echo "imarith @ldata.1.tbl - sky1 @ldata.1.tbl" > cl.com
if( $NFRAME > 1 ) then
  echo "imarith @ldata.2.tbl - sky2 @ldata.2.tbl" >> cl.com
endif
echo "imdel tmp2*.imh,tmp2*.fits" >> cl.com
echo "logout" >> cl.com   
cat cl.com | cl >>& $CLLOG  
if( $NFRAME > 1 ) then 
  ls sky2.???.imh > tmp.tbl
  sed -e 's/sky2/tmp2/g' tmp.tbl > tmp2.tbl
  echo "imarith @tmp.tbl - sky2 @tmp2.tbl" > cl.com
else
  ls sky1.???.imh > tmp.tbl
  sed -e 's/sky1/tmp2/g' tmp.tbl > tmp2.tbl
  echo "imarith @tmp.tbl - sky1 @tmp2.tbl" > cl.com
endif
echo "logout" >> cl.com   
cat cl.com | cl >>& $CLLOG  

echo "### Pattern Matching ###"
echo "### Pattern Matching ###" >> $CLLOG 
echo "del tmp.sigma" > cl.com 
echo "imstatrej tmp2.001[300:600,300:600] 3 > tmp.sigma" >> cl.com
echo "logout" >> cl.com   
cat cl.com | cl >>& $CLLOG  
set SIGMA=`gawk '{if(NR==1)print($3*1.5)}' tmp.sigma` 

echo " # Detecting Objects in Each Position #"
echo " # Detecting Objects in Each Position #" >> $CLLOG 
echo "noao" > cl.com 
echo "digiphot" >> cl.com 
echo "apphot" >> cl.com
foreach filename (`cat tmp2.tbl`) 
  set COORDFILE=`echo $filename | sed 's/imh/coo/g'` 
  if( -f $COORDFILE ) then 
    \rm $COORDFILE  
  endif 
  echo "imdel tmpmed" >> cl.com
  echo "imcopy "$filename" tmpmed" >> cl.com
  if ($DITH > 0) then
    echo "imshift tmpmed tmpmed "$DITH" "$DITH" boundary=constant">> cl.com
    echo "imshift tmpmed tmpmed -"$DITH2" 0 boundary=constant" >> cl.com
    echo "imshift tmpmed tmpmed 0 -"$DITH2" boundary=constant" >> cl.com
    echo "imshift tmpmed tmpmed "$DITH" "$DITH" boundary=constant" >> cl.com
  endif
  echo "median "$filename" tmpmed 3 3" >> cl.com
  echo "daofind tmpmed output="$COORDFILE" fwhmpsf=8 thresho=4 verify- sigma="$SIGMA" sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> cl.com 
  echo "display $filename 1 fi+" >> cl.com 
  echo "tvmark 1 $COORDFILE number+" >> cl.com 
end 
echo "imdel tmpmed" >> cl.com
echo "logout" >> cl.com    
cat cl.com | cl >>& $CLLOG 

echo "# Pattern Matching 1 #"
echo "# Pattern Matching 1 #" >> $CLLOG 
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set COORDFILE=`echo $filename | sed 's/imh/coo/g'`  
  set COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g'`  
  set XOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($1)}'`
  set YOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($2)}'`
  grep -v "^#" $COORDFILE | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $COORDFILE2
  @ i = $i + 1
end

echo " Executing matchpat"
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g'`  
#  echo $COORDFILE2 $i
  if ( $i == 1 ) then
    set basefile=$COORDFILE2
    echo "0 0" > tmp.offset2
  else
    ./matchpat $basefile $COORDFILE2 40 >> tmp.offset2
  endif
  @ i = $i + 1
end

if ( $DEBUG == 1 ) then
  foreach filename (`cat tmp2.tbl`)  
    set COORDFILE2=`echo $filename | sed 's/.imh/2.coo/g'`  
    \rm $COORDFILE2
  end
endif

mv tmp.offset1 tmp.offsettmp

paste tmp.offsettmp tmp.offset2 | gawk '{print($1+$3,$2+$4)}' > tmp.offset1

echo "# Pattern Matching 2 #"
echo "# Pattern Matching 2 #" >> $CLLOG
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set COORDFILE=`echo $filename | sed 's/imh/coo/g'`  
  set COORDFILE3=`echo $filename | sed 's/.imh/3.coo/g'`  
  set XOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($1)}'`
  set YOFFSET=`sed -n ${i}p tmp.offset1 | gawk '{print($2)}'`
  grep -v "^#" $COORDFILE | gawk -v dx=$XOFFSET -v dy=$YOFFSET '{print($1+dx,$2+dy)}' > $COORDFILE3
  if ( $DEBUG == 1 ) then 
    \rm $COORDFILE
  endif
  @ i = $i + 1
end

echo " Executing matchpat"
@ i = 1 
foreach filename (`cat tmp2.tbl`)  
  set COORDFILE3=`echo $filename | sed 's/.imh/3.coo/g'`  
  if ( $i == 1 ) then
    set basefile=$COORDFILE3
    echo "0 0" > tmp.offset2
  else
    ./matchpat $basefile $COORDFILE3 5 >> tmp.offset2
  endif
  @ i = $i + 1
end

if ( $DEBUG == 1 ) then 
  foreach filename (`cat tmp2.tbl`)  
    set COORDFILE3=`echo $filename | sed 's/.imh/3.coo/g'`  
    \rm $COORDFILE3
  end
endif

paste tmp.offset1 tmp.offset2 | gawk '{print($1+$3,$2+$4)}' > tmp.offset

echo "### Shift and Combine ###"
echo "### Shift and Combine ###" >> $CLLOG
echo "imdel tmp.tot*" > cl.com
echo "imcombine @tmp2.tbl tmp.tot combine=median reject=sigclip outtype='real' lsigma=2 hsigma=2 offset=tmp.offset" >> cl.com
echo "logout" >> cl.com   
cat cl.com | cl >>& $CLLOG  
\cp tmp.offset offset.$HEADER
if ( -f RESULTS/offset.$HEADER ) then
\rm RESULTS/offset.$HEADER
endif
mv offset.$HEADER RESULTS/

echo "### Object Detection ###"
echo "### Object Detection ###" >> $CLLOG
#calculate background sigma
echo "del tmp.sigma" > cl.com
echo "imstatrej tmp.tot[300:600,300:600] 3 > tmp.sigma" >> cl.com
echo "logout" >> cl.com   
cat cl.com | cl >>& $CLLOG  

set SIGMA=`gawk '{if(NR==1)print($3*2)}' tmp.sigma`
if ( $DEBUG == 1 ) then
  \rm tmp.sigma
endif

echo "noao" > cl.com
echo "digiphot" >> cl.com
echo "apphot" >> cl.com
echo "del tmp.tot.coo" >> cl.com
echo "imdel tmp1.fits,tmp1.imh" >> cl.com
echo "gauss tmp.tot tmp1 2.5" >> cl.com
if ($DITH > 0) then
  echo "imshift tmp1 tmp1 "$DITH" "$DITH" boundary=constant">> cl.com
  echo "imshift tmp1 tmp1 -"$DITH2" 0 boundary=constant" >> cl.com
  echo "imshift tmp1 tmp1 0 -"$DITH2" boundary=constant" >> cl.com
  echo "imshift tmp1 tmp1 "$DITH" "$DITH" boundary=constant" >> cl.com
endif
echo "displ tmp.tot 1 fi+" >> cl.com
echo "daofind tmp1 output="tmp.tot.coo" fwhmpsf=15 thresho=4 verify- sigma="$SIGMA "sharplo=-0.5 sharphi=5 roundlo=-5 roundhi=5" >> cl.com
echo "tvmark 1 tmp.tot.coo number+" >> cl.com
echo "imdel RESULTS/$OUTIM" >> cl.com
echo "imcopy tmp.tot RESULTS/$OUTIM" >> cl.com
echo "imdel RESULTS/$HEADER.sky1.fits" >> cl.com
echo "imcopy sky1.imh RESULTS/$HEADER.sky1.fits" >> cl.com
if ( $NFRAME > 1 ) then
  echo "imdel RESULTS/$HEADER.sky2.fits" >> cl.com
  echo "imcopy sky2.imh RESULTS/$HEADER.sky2.fits" >> cl.com
endif
if ( $DEBUG == 1 ) then
  echo "imdel tmp.tot,tmp1" >> cl.com
  echo "imdel sky1,sky1.???,sky2,sky2.???" >> cl.com
endif
echo "logout" >> cl.com   
cat cl.com | cl >>& $CLLOG  
./offsettbl tmp.tot.coo RESULTS/offset.$HEADER MASK/$HEADER
if ( $DEBUG == 1 ) then 
  \rm tmp.tot.coo
endif

echo "### $0 Finished ###"
echo "### $0 Finished ###" >> $CLLOG
if ( $DEBUG == 1 ) then 
  echo "imdel @tmp2.tbl" > cl.com
  echo "logout" >> cl.com    
  cat cl.com | cl >>& $CLLOG   
  \rm tmp2.tbl
endif
echo $NPOINT" points dithering."
echo "Output Image is "RESULTS/$OUTIM
echo "Sky Image is "RESULTS/$HEADER.sky1.fits
if ( $NFRAME > 1 ) then
  echo "             "RESULTS/$HEADER.sky2.fits
endif
echo "Offset pattern file is saved as "RESULTS/offset.$HEADER
echo "Offset pattern is : "
cat RESULTS/offset.$HEADER
echo "Sigma value for daofind is " $SIGMA
echo "Mask of objcts are saved as"
ls MASK/$HEADER.0??.tbl | cat
echo "IRAF log written in $CLLOG"
echo "."
